@@ -44,19 +44,38 @@
 namespace google {
 namespace protobuf {
 
+std::string MessageLite::GetTypeName() const {
+  auto* data = GetClassData();
+  ABSL_DCHECK(data != nullptr);
+
+  if (!data->is_lite) {
+    // For !LITE messages, we use the descriptor method function.
+    return data->full().descriptor_methods->get_type_name(*this);
+  }
+
+  // For LITE messages, the type name is a char[] just beyond ClassData.
+  return reinterpret_cast<const char*>(data) + sizeof(ClassData);
+}
+
 void MessageLite::OnDemandRegisterArenaDtor(Arena* arena) {
   if (arena == nullptr) return;
   auto* data = GetClassData();
-  if (data != nullptr && data->on_demand_register_arena_dtor != nullptr) {
+  ABSL_DCHECK(data != nullptr);
+
+  if (data->on_demand_register_arena_dtor != nullptr) {
     data->on_demand_register_arena_dtor(*this, *arena);
   }
 }
 
-const MessageLite::ClassData* MessageLite::GetClassData() const {
-  return nullptr;
-}
-
 std::string MessageLite::InitializationErrorString() const {
+  auto* data = GetClassData();
+  ABSL_DCHECK(data != nullptr);
+
+  if (!data->is_lite) {
+    // For !LITE messages, we use the descriptor method function.
+    return data->full().descriptor_methods->initialization_error_string(*this);
+  }
+
   return "(cannot determine missing fields for lite message)";
 }
 
@@ -64,13 +83,15 @@ std::string MessageLite::DebugString() const {
   return absl::StrCat("MessageLite at 0x", absl::Hex(this));
 }
 
-int MessageLite::GetCachedSize() const {
-  auto* cached_size = AccessCachedSize();
-  if (PROTOBUF_PREDICT_FALSE(cached_size == nullptr)) return ByteSize();
-  return cached_size->Get();
-}
+int MessageLite::GetCachedSize() const { return AccessCachedSize().Get(); }
 
-internal::CachedSize* MessageLite::AccessCachedSize() const { return nullptr; }
+internal::CachedSize& MessageLite::AccessCachedSize() const {
+  auto* data = GetClassData();
+  ABSL_DCHECK(data != nullptr);
+  ABSL_DCHECK(data->cached_size_offset != 0);
+  return *reinterpret_cast<internal::CachedSize*>(const_cast<char*>(
+      reinterpret_cast<const char*>(this) + data->cached_size_offset));
+}
 
 namespace {
 
@@ -230,7 +251,7 @@ bool MessageLite::MergeFromImpl(io::CodedInputStream* input,
   if (PROTOBUF_PREDICT_FALSE(!ptr)) return false;
   ctx.BackUp(ptr);
   if (!ctx.EndedAtEndOfStream()) {
-    ABSL_DCHECK_NE(ctx.LastTag(), 1);  // We can't end on a pushed limit.
+    ABSL_DCHECK_NE(ctx.LastTag(), 1u);  // We can't end on a pushed limit.
     if (ctx.IsExceedingLimit(ptr)) return false;
     input->SetLastTag(ctx.LastTag());
   } else {
@@ -630,11 +651,6 @@ void GenericTypeHandler<MessageLite>::Merge(const MessageLite& from,
                                             MessageLite* to) {
   to->CheckTypeAndMergeFrom(from);
 }
-template <>
-void GenericTypeHandler<std::string>::Merge(const std::string& from,
-                                            std::string* to) {
-  *to = from;
-}
 
 // Non-inline variants of std::string specializations for
 // various InternalMetadata routines.
